<?php
/*
 * @package Joomla 1.5
 * @copyright Copyright (C) 2005 Open Source Matters. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 *
 * @component Phoca Gallery
 * @copyright Copyright (C) Jan Pavelka www.phoca.cz
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

jimport( 'joomla.application.component.view');

/**
 * Phoca Gallery View Categories class
 */
class PhocaGalleryViewCategories extends JView
{
	/**
	 * Method to display the Phoca Gallery View Categories (category list)
	 * @param template $tpl
	 */
	function display($tpl = null)
	{		
		global $mainframe;
		$db 		=& JFactory::getDBO();
		$model		= &$this->getModel();
		$document	= & JFactory::getDocument();
		$user 		=& JFactory::getUser();
		$params		= &$mainframe->getParams();
		// CSS
		$document->addCustomTag("<!--[if IE]>\n<link rel=\"stylesheet\" href=\"".JURI::base(true)."/components/com_phocagallery/assets/phocagalleryieall.css\" type=\"text/css\" />\n<![endif]-->");
		$document->addStyleSheet(JURI::base(true).'/components/com_phocagallery/assets/phocagallery.css');
		
		// PARAMS - Display or hide image beside the category name - Image size
		$image_categories_size = $params->get( 'image_categories_size', 4 );
		// PARAMS - Hide categories (hide categories in category list which we don't want to see in list)
		$hide_categories = $params->get( 'hide_categories', '' );
		
		// PARAMS - Thumbnails size
		// Get thumbnail sizes from params
		$medium_image_width 	= $params->get( 'medium_image_width', 100 );
		$medium_image_height 	= $params->get( 'medium_image_height', 100 );
		$small_image_width 		= $params->get( 'small_image_width', 50 );
		$small_image_height 	= $params->get( 'small_image_height', 50 );
		
		$medium_image_height	= $medium_image_height + 18;
		$medium_image_width 	= $medium_image_width + 18;
		$small_image_width		= $small_image_width +18;
		$small_image_height		= $small_image_height +18;
		
		switch ($image_categories_size)
		{	
			case 4:
			case 6:			
				$imageBg = 'background: url(\''.JURI::base(true).'/components/com_phocagallery/assets/images/shadow3.'.PhocaGalleryHelperFront::getFormatIcon().'\') 50% 50% no-repeat;height:'.$small_image_height	.'px;width:'.$small_image_width.'px;';
			break;
			
			case 5:
			case 7:
				$imageBg = 'background: url(\''.JURI::base(true).'/components/com_phocagallery/assets/images/shadow1.'.PhocaGalleryHelperFront::getFormatIcon().'\') 50% 50% no-repeat;height:'.$medium_image_height	.'px;width:'.$medium_image_width.'px;';
			break;
			
			case 1:
			case 3:
				//$imageBg = 'height:'.$medium_image_height	.'px;width:'.$medium_image_width.'px;';
				$imageBg = 'width:'.$medium_image_width.'px;';
			break;
			
			case 0:
			case 2:
			default:
				//$imageBg = 'height:'.$small_image_height	.'px;width:'.$small_image_width.'px;';
				$imageBg = 'width:'.$small_image_width.'px;';
			break;
		}
		
		// PARAMS
		$phocagallery_width 		= $params->get( 'phocagallery_width', '' );
		$display_subcategories 		= $params->get( 'display_subcategories', 1 );
		$display_empty_categories 	= $params->get( 'display_empty_categories', 0 );
		$categories_columns 		= $params->get( 'categories_columns', 1 );
		$display_phoca_info 		= $params->get( 'display_phoca_info', 1 );
		
		$phocaInfoCode 	= PhocaGalleryHelper::getPhocaInfoCode((int)$display_phoca_info);
		$categories		= $model->getData($display_subcategories, $display_empty_categories, $hide_categories);
		
		// PARMAS - Access Category - display category in category list, which user cannot access
		$display_access_category = $params->get( 'display_access_category', 1 );
		
		// Add link and unset the categories which user cannot see (if it is enabled in params)
		$unSet = 0;// If it will be unseted while access view, we must sort the keys from category array - ACCESS
	
		foreach ($categories as $key => $category) { 
			$categories[$key]->link	= JRoute::_('index.php?option=com_phocagallery&view=category&id='. $category->slug.'&Itemid='. JRequest::getVar('Itemid', 1, 'get', 'int') ); 
			$file_thumbnail			= PhocaGalleryHelperFront::displayFileOrNoImageCategories($category->filename, $image_categories_size); 
			$categories[$key]->linkthumbnailpath	= $file_thumbnail['rel'];

			// USER RIGHT - ACCESS =======================================
			$rightDisplay	= 1;
			
			if (isset($categories[$key]->params)) {
				$rightDisplay = PhocaGalleryHelper::getUserRight ($categories[$key]->params, 'accessuserid', $category->access, $user->get('aid', 0), $user->get('id', 0), $display_access_category);
			}
				
			if ($rightDisplay == 0) {
				unset($categories[$key]);
				$unSet = 1;
			}
			// ============================================================
				
		}
		
		
		
		// ACCESS - in case we unset some category from the list, we must sort the array new
		if ($unSet == 1) {
			$categories = array_values($categories);
		}

		// PARAMS - Define image tag attributes
		if ($params->get('image') != -1) {
			$attribs['align']	= $params->get('image_align');
			$attribs['hspace']	= 6;
			// Use the static HTML library to build the image tag
			$image = JHTML::_('image', 'images/stories/'.$params->get('image'), JText::_('Phoca gallery'), $attribs);
		}
		
		// PARAMS - Display or hide image beside the category name
		$display_image_categories = $params->get( 'display_image_categories', 1 );

		// ASSIGN
		$this->assignRef( 'phocainfocode' ,			$phocaInfoCode);
		$this->assignRef( 'phocagallerywidth' ,		$phocagallery_width);
		$this->assignRef('image',					$image);
		$this->assignRef('params',					$params);
		$this->assignRef('categories',				$categories);
		$this->assignRef('displayimagecategories',	$display_image_categories);
		$this->assignRef('imagebackground',			$imageBg);
		$this->assignRef('categoriescolumns',		$categories_columns);

		// PARAMS - GEO
		$display_categories_geotagging 		= $params->get( 'display_categories_geotagging', 0 );
		
		if ($display_categories_geotagging == 1) {
		
			$categories_lng 		= $params->get( 'categories_lng', '' );
			$categories_lat 		= $params->get( 'categories_lat', '' );
			$categories_zoom 		= $params->get( 'categories_zoom', 2 );
			$google_maps_api_key 	= $params->get( 'google_maps_api_key', '' );
			$categories_map_width 	= $params->get( 'categories_map_width', 500 );
			$categorires_map_height = $params->get( 'categorires_map_height', 500 );

			// if no lng and lat will be added, Phoca Gallery will try to find it in categories
			if ($categories_lat == '' || $categories_lng == '') {
				$latLng = PhocaGalleryHelper::findLatLngFromCategory($categories);
				$categories_lng = $latLng['lng'];
				$categories_lat = $latLng['lat'];
			}
		
			$this->assignRef('categories',				$categories);
			$this->assignRef('categorieslng',			$categories_lng);
			$this->assignRef('categorieslat',			$categories_lat);
			$this->assignRef('categorieszoom',			$categories_zoom);
			$this->assignRef('googlemapsapikey',		$google_maps_api_key);
			$this->assignRef('categoriesmapwidth',		$categories_map_width);
			$this->assignRef('categoriesmapheight',		$categorires_map_height);
			
			parent::display('map');
		} else {
			parent::display($tpl);
		}
	}
}
?>